@@ -1,11 +1,12 @@
 from pathlib import Path
 
+from django.apps import apps
 from django.core.management import call_command
 from django.core.management.base import BaseCommand
 
 
 class Command(BaseCommand):
-    help = "Import Objects, Properties and Relationships from a directory"
+    help = "Import objects and relationships from a directory"
 
     def add_arguments(self, parser):
         parser.add_argument("path")
@@ -28,38 +29,41 @@ class Command(BaseCommand):
         else:
             file_types = [file_type.lower() for file_type in file_types]
             for file_type in file_types:
-                if file_type not in ("property", "object", "relationship"):
+                if file_type not in ("object", "relationship"):
                     raise ValueError(f"Unknown value '{file_type}'")
 
-        # First, import properties (before objects - so when objects are
-        # imported we'll fill search_data with property data)
-        if file_types is all or "property" in file_types:
-            filenames = path.glob("*property*.csv*")
-            filenames = [str(filename.absolute()) for filename in filenames]
-            ok = call_command("import_properties", *filenames, **cmd_options) == "True"
-            if not ok:
-                return ok
-
-        # Second, import objects
+        # Import files containing objects
+        model_app = {
+            model_name.lower(): app_name
+            for app_name, app_models in apps.all_models.items()
+            for model_name, Model in app_models.items()
+        }
         if file_types is all or "object" in file_types:
-            filenames = path.glob("*object*.csv*")
-            filenames = [str(filename.absolute()) for filename in filenames]
-            ok = call_command("import_objects", *filenames, **cmd_options) == "True"
-            if not ok:
-                return ok
+            filenames = [
+                str(filename.absolute())
+                for filename in path.glob("*object*.csv*")
+            ]
+            oks = []
+            for filename in filenames:
+                model = filename.split("object")[1].split(".csv")[0].replace("-", "").replace("_", "")
+                app_name = model_app[model]
+                oks.append(call_command("import_objects", app_name, model, filename, **cmd_options) == "True")
+            if not all(oks):
+                return oks
 
-        # Finally, import relations
+        # Import files containing relationships
         if file_types is all or "relationship" in file_types:
-            relations_filenames = []
+            oks = []
             for filename in path.glob("*relationship*.csv*"):
                 relationship = filename.absolute().name.split("relationship")[1].split(".csv")[0]
                 filename = str(filename.absolute())
-                if relationship.startswith("-"):
+                if relationship.startswith("-") or relationship.startswith("_"):
                     relationship = relationship[1:]
-                if relationship.endswith("-"):
+                if relationship.endswith("-") or relationship.endswith("_"):
                     relationship = relationship[:-1]
-                relations_filenames.append(f"{relationship}--{filename}")
 
-            ok = call_command("import_relations", *relations_filenames, **cmd_options_relations) == "True"
-            if not ok:
-                return ok
+                oks.append(
+                    call_command("import_relationships", relationship, filename, **cmd_options_relations) == "True"
+                )
+                if not all(oks):
+                    return oks
