@@ -14,6 +14,7 @@ from django.db.utils import NotSupportedError
 from django.template import Context, Template
 
 from urlid_graph import settings as urlid_graph_settings
+from urlid_graph.utils import random_name
 
 from . import formatting
 
@@ -84,6 +85,11 @@ class Entity(models.Model):
     def __str__(self):
         return f"Entity {self.url}"
 
+    def get_model(self):
+        for Model in ObjectModelMixin.__subclasses__():
+            if str(Model._meta.entity_uuid) == str(self.uuid):
+                return Model
+
 
 class ObjectMixin:
     """Mixin used by Object and ObjectRepository classes"""
@@ -94,11 +100,7 @@ class ObjectMixin:
         return urljoin(self.entity.url, f"{self.uuid}/")
 
     def _get_objects(self):
-        ObjectModel = None
-        for Model in ObjectModelMixin.__subclasses__():
-            if str(Model._meta.entity_uuid) == str(self.entity.uuid):
-                ObjectModel = Model
-                break
+        ObjectModel = self.entity.get_model()
         if ObjectModel is None:
             raise ValueError("Model class not found fo entity UUID {}".format(self.entity.uuid))
 
@@ -159,15 +161,6 @@ class ObjectMixin:
         return f"Object {self.uuid}"
 
 
-class Object(ObjectMixin, models.Model):
-    # XXX: DO NOT USE! Use ObjectRepository instead
-
-    uuid = models.UUIDField(primary_key=False, blank=False, null=False, db_index=True)
-    entity = models.ForeignKey(
-        Entity, on_delete=models.DO_NOTHING, db_column="entity_uuid", db_constraint=False, db_index=True
-    )
-
-
 class PropertyQuerySet(models.QuerySet):
     def for_object(self, obj):
         return self.filter(object_id=obj.uuid)
@@ -176,55 +169,6 @@ class PropertyQuerySet(models.QuerySet):
         return self.filter(object_id__in=[obj.uuid for obj in objs])
 
 
-class Property(models.Model):
-    STR, DATE, DATETIME, FLOAT, INT, BOOL = range(1, 7)
-    TYPE_CHOICES = [
-        (STR, "str"),
-        (DATE, "date"),
-        (DATETIME, "datetime"),
-        (FLOAT, "float"),
-        (INT, "int"),
-        (BOOL, "bool"),
-    ]
-    objects = PropertyQuerySet.as_manager()
-
-    object = models.ForeignKey(
-        "ObjectRepository",
-        on_delete=models.DO_NOTHING,
-        db_column="object_uuid",
-        db_constraint=False,
-        db_index=False,
-    )
-    name = models.TextField(blank=False, null=False)
-    value = models.TextField(blank=True, null=True)
-    value_type = models.SmallIntegerField(blank=False, null=False, choices=TYPE_CHOICES)
-    source = models.TextField(blank=False, null=False)
-    value_datetime = models.DateTimeField(blank=False, null=True)
-    updated_at = models.DateTimeField(blank=True, null=True)
-
-    class Meta:
-        indexes = [
-            models.Index(fields=["object", "value_type"]),
-        ]
-
-    def __str__(self):
-        return f"Property {self.name} of {self.object.uuid}"
-
-    @property
-    def converted_value(self):
-        value = (self.value or "").strip()
-        if not value:
-            return None
-
-        property_convert_function = {
-            Property.STR: str,
-            Property.DATE: formatting.convert_date,
-            Property.DATETIME: formatting.convert_datetime,
-            Property.FLOAT: float,
-            Property.INT: int,
-            Property.BOOL: formatting.convert_bool,
-        }
-        return property_convert_function[self.value_type](value)
 
 
 class EntityConfigManager(models.Manager):
@@ -361,61 +305,76 @@ class ObjectRepositoryQuerySet(models.QuerySet):
             qs.append(obj)
         return qs
 
-    def refresh(self, concurrently=False, rebuild=False):
-        """Refresh the materialized view in which this model is based on"""
+    def index(self, Model, start_id=None, lang="pg_catalog.portuguese", batch_size=10000, callback=None):
+        """Update the FTS index using objects from `Model`"""
 
-        view_name = "urlid_graph_object_fts"
+        table_name = Model._meta.db_table
+        temp_table_name = "urlid_tmp_fts_" + random_name(16)
+        create_temp_table_sqls = [
+            (
+                f'CREATE TEMPORARY TABLE "{temp_table_name}" ("id" SERIAL, "uuid" UUID)',
+                [],
+            ),
+            (
+                dedent(
+                    f"""
+                    INSERT INTO "{temp_table_name}" ("uuid")
+                    SELECT DISTINCT object_uuid FROM "{table_name}" WHERE id >= %s
+                    """
+                ).strip(),
+                [start_id or 0],
+            ),
+            (
+                f'CREATE INDEX "idx_{random_name(20)}" ON "{temp_table_name}" ("id", "uuid")',
+                [],
+            ),
+        ]
+        row_count = []
         with connections[urlid_graph_settings.DJANGO_DATABASE].cursor() as cursor:
-            if rebuild:  # First, create/recreate the view
-                drop_query = 'DROP MATERIALIZED VIEW IF EXISTS "{view_name}" CASCADE'.format(view_name=view_name)
-                cursor.execute(drop_query)
+            for sql, args in create_temp_table_sqls:
+                cursor.execute(sql, args)
+                row_count.append(cursor.rowcount)
+        total_objects = row_count[1]
 
-                select_queries = indent(
-                    "\nUNION ALL\n".join(
-                        Model._materialized_view_sql()
-                        for Model in ObjectModelMixin.__subclasses__()
-                    ),
-                    "  "
+        string_agg = " || ' ' || ".join(
+            f"COALESCE(STRING_AGG({field_name}, ' '), '')"
+            for field_name in Model._meta.search_fields
+        )
+        insert_sql = dedent(
+            f"""
+            INSERT INTO "{ObjectRepository._meta.db_table}" (uuid, entity_id, search_data)
+                SELECT
+                    object_uuid,
+                    %s::uuid,
+                    to_tsvector(
+                        %s,
+                        {string_agg}
+                    )
+                FROM "{table_name}"
+                WHERE object_uuid IN (
+                    SELECT "uuid"
+                    FROM "{temp_table_name}"
+                    WHERE id >= %s AND id < %s
                 )
-                create_query = dedent(
-                    """
-                    CREATE MATERIALIZED VIEW "{view_name}" AS
-                    {select_queries}
-                    WITH NO DATA
-                    """
-                ).strip().format(view_name=view_name, select_queries=select_queries)
-                cursor.execute(create_query)
+                GROUP BY object_uuid
+            ON CONFLICT (uuid) DO UPDATE SET search_data = EXCLUDED.search_data
+            """
+        ).strip()
 
-                indices_queries = [
-                    dedent(
-                        """
-                        CREATE UNIQUE INDEX IF NOT EXISTS "idx_urlid_object_fts_uuid" ON "{view_name}" (
-                            "uuid"
-                        )
-                        """
-                    ).strip().format(view_name=view_name),
-                    dedent(
-                        """
-                        CREATE INDEX IF NOT EXISTS "idx_urlid_object_fts_vector" ON "{view_name}" USING gin (
-                            "search_data"
-                        )
-                        """
-                    ).strip().format(view_name=view_name),
-                ]
-                for query in indices_queries:
-                    cursor.execute(query)
+        counter = 0
+        for object_id in range(1, total_objects + 1, batch_size):
+            args = (Model._meta.entity_uuid, lang, object_id, object_id + batch_size)
+            with connections[urlid_graph_settings.DJANGO_DATABASE].cursor() as cursor:
+                cursor.execute(insert_sql, args)
+                counter += cursor.rowcount
+                if callback is not None:
+                    callback(done=counter, total=total_objects)
 
-            regular_query = f"REFRESH MATERIALIZED VIEW {view_name}"
-            concurrently_query = f"REFRESH MATERIALIZED VIEW CONCURRENTLY {view_name}"
-            if not concurrently:
-                cursor.execute(regular_query)
-            else:
-                try:
-                    cursor.execute(concurrently_query)
-                except NotSupportedError:
-                    # Materialized view not populated, so CONCURRENTLY cannot
-                    # be used
-                    cursor.execute(regular_query)
+        drop_temp_table_sql = f'DROP TABLE "{temp_table_name}"'
+        with connections[urlid_graph_settings.DJANGO_DATABASE].cursor() as cursor:
+            cursor.execute(drop_temp_table_sql)
+
+        return counter
 
     def search(self, search_query, config=urlid_graph_settings.SEARCH_LANGUAGE):
         """Full-Text Search in object's search_data"""
@@ -464,13 +423,11 @@ class ObjectRepository(ObjectMixin, models.Model):
 
     uuid = models.UUIDField(primary_key=True)
     entity = models.ForeignKey(
-        Entity, on_delete=models.DO_NOTHING, db_column="entity_uuid", db_constraint=False, db_index=False
+        Entity, on_delete=models.DO_NOTHING, db_constraint=False, db_index=False
     )
     search_data = SearchVectorField(null=True)
 
     class Meta:
-        db_table = "urlid_graph_object_fts"
-        managed = False
         indexes = [GinIndex(fields=["search_data"])]
         unique_together = ["entity", "uuid"]
 
@@ -499,25 +456,5 @@ class ObjectModelMixin(models.Model):
             if field.name != "search_data"
         }
 
-    @classmethod
-    def _materialized_view_sql(cls, lang="pg_catalog.portuguese"):
-        string_agg = " || ' ' || ".join(
-            f"COALESCE(STRING_AGG({field_name}, ' '), '')"
-            for field_name in cls._meta.search_fields
-        )
-        return dedent(
-            f"""
-            SELECT
-                object_uuid AS uuid,
-                '{cls._meta.entity_uuid}'::uuid AS entity_uuid,
-                to_tsvector(
-                    '{lang}',
-                    {string_agg}
-                ) AS search_data
-            FROM {cls._meta.db_table}
-            GROUP BY object_uuid
-            """
-        ).strip()
-
     class Meta:
         abstract = True
